/*
 * This file is part of
 *    ivil - Interactive Verification on Intermediate Language
 *
 * Copyright (C) 2009-2012 Karlsruhe Institute of Technology
 *
 * The system is protected by the GNU General Public License.
 * See LICENSE.TXT (distributed with this file) for details.
 */
package de.uka.iti.pseudo.environment;

import nonnull.NonNull;

/**
 * Any class implementing this interface allows to store instances in a map
 * indexed by a key. The retrieval can then be done by the key which is usually
 * faster.
 *
 * @param <T> The type of the key which is returned by a class.
 */
public interface Mappable<T> {

    /**
     * Gets the key under which this object can be retrieved from a map. It must
     * be unique amongst different objects which can end up in the same map.
     *
     * @return a nonnull reference
     */
    public @NonNull T getKey();

}
