/*
 * This file is part of
 *    ivil - Interactive Verification on Intermediate Language
 *
 * Copyright (C) 2009-2012 Karlsruhe Institute of Technology
 *
 * The system is protected by the GNU General Public License.
 * See LICENSE.TXT (distributed with this file) for details.
 */
package de.uka.iti.pseudo.parser.term;

import java.util.List;

import nonnull.NonNull;
import de.uka.iti.pseudo.parser.ASTVisitException;
import de.uka.iti.pseudo.parser.ASTVisitor;
import de.uka.iti.pseudo.parser.Token;
import de.uka.iti.pseudo.term.creation.Typing;
import de.uka.iti.pseudo.util.Pair;

/**
 * The AST class for binder terms generated by the
 * {@link de.uka.iti.pseudo.parser.Parser}.
 *
 * Bound variables are kept as pairs of Token and ASTType.
 */
public class ASTBinderTerm extends ASTTerm {

    private final Typing[] variableTypings;
    private final Token binderToken;

    // the list is reversed: first element in list is last in this enumeration
    private final List<Pair<Token, ASTType>> boundVars;

    // Checkstyle: IGNORE JavadocMethod - AST creation is obvious
    public ASTBinderTerm(Token binderToken,
            List<Pair<Token, ASTType>> boundVars,
            List<ASTTerm> subterms) {
        super(subterms);
        this.binderToken = binderToken;
        this.boundVars = boundVars;
        this.variableTypings = new Typing[boundVars.size()];

        for (Pair<Token, ASTType> var : boundVars) {
            ASTType type = var.snd();
            if(type != null) {
                addChild(type);
            }
        }
    }

    @Override
    public void visit(ASTVisitor v) throws ASTVisitException {
        v.visit(this);
    }

    public final Token getBinderToken() {
        return binderToken;
    }

    // Checkstyle: IGNORE JavadocMethod - this is a getter
    public final int countBoundVariables() {
        return boundVars.size();
    }

    // Checkstyle: IGNORE JavadocMethod - this is a getter
    public final ASTType getVariableType(int index) {
        return boundVars.get(index).snd();
    }

    // Checkstyle: IGNORE JavadocMethod - this is a getter
    public final Token getVariableToken(int index) {
        return boundVars.get(index).fst();
    }

    @Override
    public Token getLocationToken() {
        return binderToken;
    }
    // Checkstyle: IGNORE JavadocMethod - this is a getter
    public @NonNull Typing getVariableTyping(int index) {
        return variableTypings[index];
    }
    // Checkstyle: IGNORE JavadocMethod - this is a setter
    public void setVariableTyping(int index, Typing variableTyping) {
        variableTypings[index] = variableTyping;
    }

}
