package de.uka.iti.pseudo.util;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

// TODO DOC
public class ResetHashMap<K,V> implements Map<K, V> {

    private HashMap<K, V> map;
    private HashMap<K, V> savedMap;

    public ResetHashMap() {
        this.map = new HashMap<K, V>();
    }

    public void mark() {
        savedMap = null;
    }

    public boolean reset() {
        if(savedMap != null) {
            map = savedMap;
            savedMap = null;
            return true;
        } else {
            return false;
        }
    }

    public boolean hasCopy() {
        return savedMap != null;
    }

    private void saveMap() {
        if(savedMap == null) {
            savedMap = (HashMap<K,V>)map.clone();
        }
    }


    //
    // --- delegating to the underlying hash map
    //

    /**
     * @return
     * @see java.util.HashMap#size()
     */
    public int size() {
        return map.size();
    }

    /**
     * @return
     * @see java.util.HashMap#isEmpty()
     */
    public boolean isEmpty() {
        return map.isEmpty();
    }

    /**
     * @param key
     * @return
     * @see java.util.HashMap#get(java.lang.Object)
     */
    public V get(Object key) {
        return map.get(key);
    }

    /**
     * @return
     * @see java.util.AbstractMap#hashCode()
     */
    public int hashCode() {
        return map.hashCode();
    }

    /**
     * @param o
     * @return
     * @see java.util.AbstractMap#equals(java.lang.Object)
     */
    public boolean equals(Object o) {
        return map.equals(o);
    }

    /**
     * @param key
     * @return
     * @see java.util.HashMap#containsKey(java.lang.Object)
     */
    public boolean containsKey(Object key) {
        return map.containsKey(key);
    }

    /**
     * @return
     * @see java.util.AbstractMap#toString()
     */
    public String toString() {
        return map.toString();
    }

    /**
     * @param value
     * @return
     * @see java.util.HashMap#containsValue(java.lang.Object)
     */
    public boolean containsValue(Object value) {
        return map.containsValue(value);
    }

    /**
     * @return
     * @see java.util.HashMap#keySet()
     */
    public Set<K> keySet() {
        return map.keySet();
    }

    /**
     * @return
     * @see java.util.HashMap#values()
     */
    public Collection<V> values() {
        return map.values();
    }

    /**
     * @return
     * @see java.util.HashMap#entrySet()
     */
    public Set<java.util.Map.Entry<K, V>> entrySet() {
        return map.entrySet();
    }

    /**
     *
     * @see java.util.HashMap#clear()
     */
    public void clear() {
        saveMap();
        map.clear();
    }

    /**
     * @param key
     * @return
     * @see java.util.HashMap#remove(java.lang.Object)
     */
    public V remove(Object key) {
        saveMap();
        return map.remove(key);
    }

    /**
     * @param m
     * @see java.util.HashMap#putAll(java.util.Map)
     */
    public void putAll(Map<? extends K, ? extends V> m) {
        saveMap();
        map.putAll(m);
    }

    /**
     * @param key
     * @param value
     * @return
     * @see java.util.HashMap#put(java.lang.Object, java.lang.Object)
     */
    public V put(K key, V value) {
        saveMap();
        return map.put(key, value);
    }
}
