/*
 * This file is part of
 *    ivil - Interactive Verification on Intermediate Language
 *
 * Copyright (C) 2009-2010 Universitaet Karlsruhe, Germany
 *    written by Mattias Ulbrich
 *
 * The system is protected by the GNU General Public License.
 * See LICENSE.TXT (distributed with this file) for details.
 */
package de.uka.iti.pseudo.util;

import junit.framework.TestCase;

public class TestResetHashMap extends TestCase {


    public void testPutAndReset() throws Exception {
        ResetHashMap<String, String> rhm = new ResetHashMap<String, String>();

        rhm.put("11", "11");
        rhm.mark();
        rhm.put("11", "22");
        rhm.put("22", "22");
        assertEquals("22", rhm.get("11"));
        assertEquals("22", rhm.get("22"));
        assertTrue(rhm.hasCopy());

        boolean resetResult = rhm.reset();
        assertTrue(resetResult);

        assertEquals("11", rhm.get("11"));
        assertFalse(rhm.containsKey("22"));
        assertFalse(rhm.hasCopy());
    }

    public void testRemoveAndReset() throws Exception {
        ResetHashMap<String, String> rhm = new ResetHashMap<String, String>();

        rhm.put("11", "11");
        rhm.mark();

        assertEquals("11", rhm.get("11"));
        assertFalse(rhm.hasCopy());
        rhm.remove("11");

        assertFalse(rhm.containsKey("11"));
        assertTrue(rhm.hasCopy());

        boolean resetResult = rhm.reset();
        assertTrue(resetResult);

        assertEquals("11", rhm.get("11"));
        assertFalse(rhm.hasCopy());
    }

    public void testResetFalse() throws Exception {

        ResetHashMap<String, String> rhm = new ResetHashMap<String, String>();
        assertFalse(rhm.hasCopy());
        assertFalse(rhm.reset());
    }


}
